﻿using System;
using System.Globalization;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MSIS.webservices.schedule;

namespace MSIS
{
    public partial class Schedule : Form
    {
        CultureInfo cCulture = CultureInfo.CurrentCulture;
        DateTime currentDate;
        int weekNumber;
        int selTabIdx;
        string username; // username of logged in user
        enum CalendarView { Lectures, Exercises, Overview };
        CalendarView currentView = CalendarView.Lectures;
        Panel[] viewPanels;
        MenuItem[] viewMenuItems;
        int tmpClickedAppointmentIdx = -1;

        // Remote fields/objects
        ScheduleService scheduleService;
        Appointment[] appointments; // all calendar appointments for user

        public Schedule(string username)
        {
            InitializeComponent();
            Text = AppSettings.AppTitle;

            this.username = username;
            try
            {
                scheduleService = new webservices.schedule.ScheduleService();
                scheduleService.Url = AppSettings.WebServiceUrl + "/" + AppSettings.ScheduleServiceName;
            }
            catch
            {
                Main.ShowConnectionError();
                Close();
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
            viewPanels = new Panel[] { panelLectures, panelExercises, panelWeekOverview };
            viewMenuItems = new MenuItem[] { menuShowLectures, menuShowExercises, menuShowWeekOverview };
            currentDate = DateTime.Today;
            if (AppSettings.ShowHints)
            {
                MessageBox.Show("Click on an appointment to show the location on the map.", "Tip", MessageBoxButtons.OK,
                    MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }
        }

        private void Schedule_Load(object sender, EventArgs e)
        {
            try
            {
                appointments = scheduleService.GetLectures(username);
            }
            catch (Exception es)
            {
                Main.ShowConnectionError();
            }

            if (currentView == CalendarView.Lectures)
            {
                ToggleDateLabels(false);
                showView(panelLectures);
                lblHeader.Text = "My Schedule (lectures)";
            }
            else if (currentView == CalendarView.Exercises)
            {
                ToggleDateLabels(false);
                showView(panelExercises);
                lblHeader.Text = "My Schedule (exercises)";
            }
            else if (currentView == CalendarView.Overview)
            {
                CreateWeeklySummary();
                showView(panelWeekOverview);
                lblHeader.Text = "Weekly Summary";
            }
            /*
            currentDate = DateTime.Today;
            SetDateTab(currentDate);
            */
        }

        private void menuItemReturn_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            // run only if target tab is different from current tab
            if (selTabIdx != (sender as TabControl).SelectedIndex)
            {
                currentDate = currentDate.AddDays((sender as TabControl).SelectedIndex - selTabIdx);
                SetDateTab(currentDate);
            }
        }

        private void SetDateTab(DateTime date)
        {
            // Workaround for 'week 53' bug
            DateTime dateFixed = (date.DayOfWeek >= System.DayOfWeek.Monday && date.DayOfWeek <= System.DayOfWeek.Wednesday) ? date.AddDays(3) : date;
            weekNumber = cCulture.Calendar.GetWeekOfYear(dateFixed, CalendarWeekRule.FirstFourDayWeek, System.DayOfWeek.Monday);
            // Update date label
            lblDate.Text = "(Week " + weekNumber + " - " + date.ToString("MMMM", CultureInfo.CreateSpecificCulture("en-US")) + " " + date.Day + ", " + date.Year + ")";

            // Select correct day tab
            int dayOfWeek = (int)date.DayOfWeek - 1;
            selTabIdx = (dayOfWeek < 0) ? 6 : dayOfWeek;
            tabControl.SelectedIndex = selTabIdx;
            if (date == DateTime.Today && !tabControl.TabPages[selTabIdx].Text.Contains("today"))
            {
                tabControl.TabPages[selTabIdx].Text += " (today)";
            }

            // Load appointments for current date/day tab,
            // first clear all appointment labels from panels
            panelLectures.Controls.Clear();
            panelExercises.Controls.Clear();

            if (appointments != null)
            {
                for (int i = 0; i < appointments.Length; i++)
                {
                    Appointment a = appointments[i];

                    // Add only appointments where date and week number match
                    bool thisWeek = Array.IndexOf<int>(a.Weeks, weekNumber) != -1 ? true : false;
                    if ((int)a.Day == (int)date.DayOfWeek && thisWeek)
                    {
                        AddAppointment(a, i);
                    }
                }
            }
        }

        private void ToggleDateLabels(bool enable)
        {
            //lblDate.Visible = !enable;
            lblMon.Visible = enable;
            lblTue.Visible = enable;
            lblWed.Visible = enable;
            lblThu.Visible = enable;
            lblFri.Visible = enable;
            tabControl.Enabled = !enable;

            if (!enable)
            {
                SetDateTab(currentDate);
            }
        }

        private void CreateWeeklySummary()
        {
            if (appointments == null)
            {
                return;
            }

            ToggleDateLabels(true);

            float scalingFactorX = AutoScaleDimensions.Width / 96.0F;
            float scalingFactorY = AutoScaleDimensions.Height / 96.0F;

            for (int i = 0; i < appointments.Length; i++)
            {
                Appointment appointment = appointments[i];

                bool thisWeek = Array.IndexOf<int>(appointment.Weeks, weekNumber) != -1 ? true : false;
                if (thisWeek)
                {
                    // Label for week summary
                    LinkLabel aptLabelOverview = new LinkLabel();
                    aptLabelOverview.BackColor = Color.FromArgb(77, 108, 131);
                    aptLabelOverview.ForeColor = Color.White;
                    aptLabelOverview.Font = new Font("Tahoma", 7.0f, FontStyle.Bold);
                    aptLabelOverview.Anchor = AnchorStyles.Top | AnchorStyles.Left;
                    aptLabelOverview.Name = "idx_" + i; // index of appointment
                    aptLabelOverview.Click += new EventHandler(aptLabel_Click);
                    aptLabelOverview.TextAlign = ContentAlignment.TopCenter;

                    // calculate height of label based on appointment duration
                    int lblHeight = Convert.ToInt32((25 / 60.0) * appointment.Duration) + 7;

                    // calculate position of label based on starting time of appointment
                    int lblYCoord = Convert.ToInt32((appointment.StartHour - 8) * 25 + appointment.StartMinute * (25 / 60.0));

                    int xPos = ((int)appointment.Day - 1) * 8 + ((int)appointment.Day - 1) * 30 + 6;
                    aptLabelOverview.Bounds = new Rectangle(xPos, lblYCoord, 28, lblHeight);
                    aptLabelOverview.Scale(new SizeF(scalingFactorX, scalingFactorY));
                    if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Lecture)
                    {
                        aptLabelOverview.Text = "*L*";
                    }
                    else if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Exercise)
                    {
                        aptLabelOverview.Text = "*E*";
                    }
                    else if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Other)
                    {
                        aptLabelOverview.Text = "*L/E*";
                    }
                    if (appointment.Room.Length < 6)
                    {
                        aptLabelOverview.Text += Environment.NewLine + appointment.Room;
                    }

                    // Add label to week overview
                    panelWeekOverview.Controls.Add(aptLabelOverview);
                    aptLabelOverview.BringToFront();
                }
            }
        }

        /// <summary>
        /// Add an appointment to the lecture or exercise panels.
        /// </summary>
        /// <param name="appointment">The appointment to add</param>
        /// <param name="idx">The index of the appointment</param>
        private void AddAppointment(Appointment appointment, int idx)
        {
            float scalingFactorX = AutoScaleDimensions.Width / 96.0F;
            float scalingFactorY = AutoScaleDimensions.Height / 96.0F;
            LinkLabel aptLabel = new LinkLabel();
            aptLabel.BackColor = Color.FromArgb(77, 108, 131);
            aptLabel.ForeColor = Color.White;
            aptLabel.Font = new Font("Tahoma", 7.0f, FontStyle.Bold);
            aptLabel.Anchor = AnchorStyles.Top | AnchorStyles.Left;
            aptLabel.Name = "idx_" + idx;
            aptLabel.Click += new EventHandler(aptLabel_Click);

            // calculate height of label based on appointment duration
            int lblHeight = Convert.ToInt32((25 / 60.0) * appointment.Duration) + 7;

            // calculate position of label based on starting time of appointment
            int lblYCoord = Convert.ToInt32((appointment.StartHour - 8) * 25 + appointment.StartMinute * (25 / 60.0));

            aptLabel.Bounds = new Rectangle(3, lblYCoord, panelLectures.Width - 6, lblHeight);
            // scale to current DPI
            aptLabel.Scale(new SizeF(1f, scalingFactorY));

            if (lblHeight >= 40)
            {
                aptLabel.Text = appointment.Title + Environment.NewLine;
                
                aptLabel.Text += "Room(s): " + appointment.Room;

                string startHr = (appointment.StartHour.ToString().Length == 1) ? "0" + appointment.StartHour : appointment.StartHour.ToString();
                string startMn = (appointment.StartMinute.ToString().Length == 1) ? "0" + appointment.StartMinute : appointment.StartMinute.ToString();
                string endHr = (appointment.EndHour.ToString().Length == 1) ? "0" + appointment.EndHour : appointment.EndHour.ToString();
                string endMn = (appointment.EndMinute.ToString().Length == 1) ? "0" + appointment.EndMinute : appointment.EndMinute.ToString();

                aptLabel.Text += Environment.NewLine + startHr + ":" + startMn + " - " + endHr + ":" + endMn;
            }
            else
            {
                aptLabel.Text = appointment.Title + Environment.NewLine + "Room: " + appointment.Room;
            }

            // Experimental... Workaround for overlapping appointments
            foreach (Control control in panelLectures.Controls)
            {
                if (control is Label)
                {
                    if ((control as Label).Location.Y == aptLabel.Location.Y)
                    {
                        aptLabel.Location = new Point(aptLabel.Location.X, aptLabel.Location.Y + 5);
                    }
                }
            }

            if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Lecture || appointment.AppointmentType == MSIS.webservices.schedule.Type.Other)
            {
                panelLectures.Controls.Add(aptLabel);
            }
            if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Exercise)
            {
                panelExercises.Controls.Add(aptLabel);
            }
            aptLabel.BringToFront();
            
        }

        void aptLabel_Click(object sender, EventArgs e)
        {
            LinkLabel lbl = sender as LinkLabel;
            tmpClickedAppointmentIdx = Int32.Parse(lbl.Name.Split('_')[1]);
            contextMenu.Show(this, new Point(lbl.Bounds.Right, lbl.Bounds.Bottom));
        }

        private void contextMenu_Click(object sender, EventArgs e)
        {
            Object item = sender;

            if (item == menuItemShowDetails && tmpClickedAppointmentIdx >= 0)
            {
                Appointment appointment = appointments[tmpClickedAppointmentIdx];

                string startHr = (appointment.StartHour.ToString().Length == 1) ? "0" + appointment.StartHour : appointment.StartHour.ToString();
                string startMn = (appointment.StartMinute.ToString().Length == 1) ? "0" + appointment.StartMinute : appointment.StartMinute.ToString();
                string endHr = (appointment.EndHour.ToString().Length == 1) ? "0" + appointment.EndHour : appointment.EndHour.ToString();
                string endMn = (appointment.EndMinute.ToString().Length == 1) ? "0" + appointment.EndMinute : appointment.EndMinute.ToString();
                string hours = startHr + ":" + startMn + " - " + endHr + ":" + endMn;

                string teacher = (appointment.Person != "") ? appointment.Person : "Unknown";
                string type = (appointment.AppointmentType == MSIS.webservices.schedule.Type.Other) ? "Lecture/Exercise" : appointment.AppointmentType.ToString();

                string detailsText =
                "Course: " + appointment.Title + Environment.NewLine +
                "Type: " + type + Environment.NewLine +
                "Time: " + appointment.Day + " " + hours + Environment.NewLine +
                "Room(s): " + appointment.Room + Environment.NewLine +
                "Teacher: " + teacher + Environment.NewLine +
                "Weeks: " + appointment.WeeksString;

                PopUp detailsPopUp = new PopUp(detailsText);
                detailsPopUp.Title = "Details";
                detailsPopUp.ShowDialog();
            }
            else if (item == menuItemShowMap)
            {
                string locRoom = appointments[tmpClickedAppointmentIdx].Room;
                MapWindow mapWindow = new MapWindow(locRoom, true);
            }
        }

        private void menuToggleScheduleView_Click(object sender, EventArgs e)
        {
            MenuItem menuItem = (sender as MenuItem);

            if (menuItem == menuShowExercises)
            {
                ToggleDateLabels(false);
                currentView = CalendarView.Exercises;
                showView(panelExercises);
                lblHeader.Text = "My Schedule (exercises)";
            }
            else if (menuItem == menuShowLectures)
            {
                ToggleDateLabels(false);
                currentView = CalendarView.Lectures;
                showView(panelLectures);
                lblHeader.Text = "My Schedule (lectures)";
            }
            else if (menuItem == menuShowWeekOverview)
            {
                currentView = CalendarView.Overview;
                CreateWeeklySummary();
                showView(panelWeekOverview);
                lblHeader.Text = "My Schedule (Weekly Summary)";
                // Update date label
                weekNumber = cCulture.Calendar.GetWeekOfYear(DateTime.Today, CalendarWeekRule.FirstFourDayWeek, System.DayOfWeek.Monday);
                lblDate.Text = "(Week " + weekNumber + ")";
            }
        }

        private void showView(Panel activePanel)
        {
            for (int i = 0; i < viewPanels.Length; i++)
            {
                if (viewPanels[i] == activePanel)
                {
                    viewPanels[i].Visible = true; // show correct view
                    viewMenuItems[i].Checked = true; // place checkmark on selected view menu item
                }
                else
                {
                    viewPanels[i].Visible = false;
                    viewMenuItems[i].Checked = false;
                }
            }
        }

    }
}